#!/usr/bin/env python3

import functools
import random
from contextlib import redirect_stdout


class Outsidebox(Exception):
    pass

class Islands():
    def __init__(self,side):
        self.d=side
        self.A=[]
        for i in range(2*side-1):
            self.A.append(['W']*(2*side-1));
        for i in range(side):
            for j in range(side+i,2*side-1):
                self.A[i][j]=-1
        for i in range(side,2*side-1):
            for j in range(i-side+1):
                self.A[i][j]=-1
    def set(self,i,j,v):
        c=i+self.d-1
        r=-j+self.d-1
        if (r<self.d and c >= self.d+r) or (r>=self.d and c < r-self.d+1) or (i==0 and j==0):
            print("e")
            raise Outsidebox
        self.A[r][c]=v
        
    def get(self,i,j):
        c=i+self.d-1
        r=-j+self.d-1
        if (r<self.d and c >= self.d+r) or (r>=self.d and c < r-self.d+1):
            print("e")
            raise Outsidebox
        return self.A[r][c]

    def is_inside(self,i,j):
        d=self.d
        if (i<=-d or i>=d or j<=-d or j>=d or i+j>=d or i+j<=-d):
            return False
        else:
            return True
   
    
        
    def __str__(self):
        output=repr(self.d)
        for i,w in enumerate(self.A):
            output+="\n"
            if i<self.d:
                output+=" "*((self.d)-i-1)
                output+=functools.reduce(lambda u,v:u+" "+v,w[:self.d+i])
                output+=" "*((self.d)-i-1)
            else:
                output+=" "*(i-(self.d)+1)
                output+=functools.reduce(lambda u,v:u+" "+v,w[i-self.d+1:2*self.d-1])
                output+=" "*(i-(self.d)+1)
        return output

def rand_L():
    return ("L" if bool(random.getrandbits(1)) else "F")

def rotate60(isl):
    d=isl.d
    isl2=Islands(d)
    for x in range(-d+1,d):
        for y in range(-d+1,d):
            if isl.is_inside(x,y) and not (x==0 and y==0):
                isl2.set(-y,x+y,isl.get(x,y))
    return isl2

def rotate120(isl):
    d=isl.d
    isl2=Islands(d)
    for x in range(-d+1,d):
        for y in range(-d+1,d):
            if isl.is_inside(x,y) and not (x==0 and y==0):
                isl2.set(-x-y,x,isl.get(x,y))
    return isl2


def peigne_vide(d):
    isl=Islands(d)
    for y in range(d):
        isl.set(-d+1,y,'L')
    for x in range(d):
        isl.set(-x,d-1,'L')
    for x in range(-d+1,d):
        for y in range(-d+1,d):
            if isl.is_inside(x,y) and (x+y)%2==1:
                isl.set(x,y,'L')
    for y in range(d):
        isl.set(d-1,-y,'W')
    for x in range(d):
        isl.set(x,-d+1,'W')
    return isl


def peigne_aleatoire(d):
    isl=Islands(d)
    for y in range(d):
        isl.set(-d+1,y,rand_L())
    for x in range(d):
        isl.set(-x,d-1,rand_L())
    for x in range(-d+1,d):
        for y in range(-d+1,d):
            if isl.is_inside(x,y) and (x+y)%2==1:
                isl.set(x,y,rand_L())
    for y in range(d):
        isl.set(d-1,-y,'W')
    for x in range(d):
        isl.set(x,-d+1,'W')
    return isl

def anneau(isl,r):
    for x in range(r):
        isl.set(-x,r-1,rand_L())
        isl.set(x,-r+1,rand_L())
        isl.set(-r+1,x,rand_L())
        isl.set(r-1,-x,rand_L())
        isl.set(x,r-1-x,rand_L())
        isl.set(-x,-r+1+x,rand_L())
    return isl

def anneauF(isl,r):
    for x in range(r):
        isl.set(-x,r-1,'F')
        isl.set(x,-r+1,'F')
        isl.set(-r+1,x,'F')
        isl.set(r-1,-x,'F')
        isl.set(x,r-1-x,'F')
        isl.set(-x,-r+1+x,'F')
    return isl

def anneauL(isl,r):
    for x in range(r):
        isl.set(-x,r-1,'L')
        isl.set(x,-r+1,'L')
        isl.set(-r+1,x,'L')
        isl.set(r-1,-x,'L')
        isl.set(x,r-1-x,'L')
        isl.set(-x,-r+1+x,'L')
    return isl


def groups(d,g):
    isl=Islands(d)
    for _ in range(g):
        x = random.randint(-d,d)
        y = random.randint(-d,d)
        dx = random.randint(1,2*d//3)
        dy = random.randint(1,2*d//3)
        for i in range(x,x+dx):
            for j in range(y,y+dy):
                 if isl.is_inside(i,j) and not (i==0 and j==0):
                    isl.set(i,j,rand_L())
    return isl

def groups_withring(d,g,r):
    return anneau(groups(d,g),r)


def manyrings(d,r):
    isl=Islands(d)
    for _ in range(r):
        anneau(isl,random.randint(2,d-2))
    return isl
    
def manyrings_split(d,r):
    isl=Islands(d)
    for _ in range(r):
        anneau(isl,random.randint(2,d-2))
    for x in range(1,d):
        isl.set(x,0,'W')
        isl.set(-x,0,'W')
    return isl

def tworings(d,r1,r2):
    isl=Islands(d)
    anneauL(isl,r1)
    anneauF(isl,r2)
    return isl


with open("peigne.in","w") as output:
    with redirect_stdout(output):
        list_d = [2,5,10,7,15]
        print(6*len(list_d))
        for d in list_d:
            print(peigne_vide(d))
            print(rotate60(peigne_vide(d)))
            print(rotate120(peigne_vide(d)))
            print(peigne_aleatoire(d))
            print(rotate60(peigne_aleatoire(d)))
            print(rotate120(peigne_aleatoire(d)))

with open("groups.in","w") as output:
    with redirect_stdout(output):
        list_d = [5,10,7,15,23,47]
        print(12*len(list_d))
        for d in list_d:
            print(groups(d,d//4))
            print(groups(d,2*d//4))
            print(groups(d,3*d//4))
            print(groups(d,4*d//5))
            print(rotate60(groups(d,d//4)))
            print(rotate60(groups(d,2*d//4)))
            print(rotate60(groups(d,3*d//4)))
            print(rotate60(groups(d,4*d//5)))
            print(rotate120(groups(d,d//4)))
            print(rotate120(groups(d,2*d//4)))
            print(rotate120(groups(d,3*d//4)))
            print(rotate120(groups(d,4*d//5)))

with open("groupswithring.in","w") as output:
    with redirect_stdout(output):
        list_d = [5,10,7,15,23,47]
        print(12*len(list_d))
        for d in list_d:
            print(groups_withring(d,d//4,d//2))
            print(groups_withring(d,2*d//4,d//2))
            print(groups_withring(d,3*d//4,d//2))
            print(groups_withring(d,4*d//5,d//2))
            print(rotate60(groups_withring(d,d//4,d//2)))
            print(rotate60(groups_withring(d,2*d//4,d//2)))
            print(rotate60(groups_withring(d,3*d//4,d//2)))
            print(rotate60(groups_withring(d,4*d//5,d//2)))
            print(rotate120(groups_withring(d,d//4,d//2)))
            print(rotate120(groups_withring(d,2*d//4,d//2)))
            print(rotate120(groups_withring(d,3*d//4,d//2)))
            print(rotate120(groups_withring(d,4*d//5,d//2)))

with open("manyrings.in","w") as output:
    with redirect_stdout(output):
        list_d = [7,17,9,43,15,29]
        print(12*len(list_d))
        for d in list_d:
            print(manyrings(d,d//4))
            print(manyrings(d,2*d//4))
            print(manyrings(d,3*d//4))
            print(manyrings(d,4*d//5))
            print(rotate60(manyrings(d,d//4)))
            print(rotate60(manyrings(d,2*d//4)))
            print(rotate60(manyrings(d,3*d//4)))
            print(rotate60(manyrings(d,4*d//5)))
            print(rotate120(manyrings(d,d//4)))
            print(rotate120(manyrings(d,2*d//4)))
            print(rotate120(manyrings(d,3*d//4)))
            print(rotate120(manyrings(d,4*d//5)))

with open("manyrings_split.in","w") as output:
    with redirect_stdout(output):
        list_d = [7,17,9,43,15,29]
        print(12*len(list_d))
        for d in list_d:
            print(manyrings_split(d,d//4))
            print(manyrings_split(d,2*d//4))
            print(manyrings_split(d,3*d//4))
            print(manyrings_split(d,4*d//5))
            print(rotate60(manyrings_split(d,d//4)))
            print(rotate60(manyrings_split(d,2*d//4)))
            print(rotate60(manyrings_split(d,3*d//4)))
            print(rotate60(manyrings_split(d,4*d//5)))
            print(rotate120(manyrings_split(d,d//4)))
            print(rotate120(manyrings_split(d,2*d//4)))
            print(rotate120(manyrings_split(d,3*d//4)))
            print(rotate120(manyrings_split(d,4*d//5)))

with open("starve.in","w") as output:
    with redirect_stdout(output):
        list_d = [51,9,31,26]
        print(1*len(list_d))
        for d in list_d:
            print(tworings(d,d//4,3*d//4))

with open("notstarve.in","w") as output:
    with redirect_stdout(output):
        list_d = [51,9,31,26]
        print(1*len(list_d))
        for d in list_d:
            print(tworings(d,3*d//4,d//4))
